from ..abstracts import *
from typing import Any

_YAML_INITIALIZED: bool

class Config:
    __EXTENSIONS_SUPPORTED: Final[tuple[str, ...]]
    @staticmethod
    def get_file_type() -> str: ...
    @staticmethod
    def __load_file(path: str) -> dict: ...
    @classmethod
    def load_file(cls, path: str) -> dict: ...
    @classmethod
    def try_load_file_if_exists(cls, _path: str, _default: dict = {}) -> dict: ...
    @classmethod
    def load(cls, path: str, *key: str) -> Any: ...
    @classmethod
    def load_internal_file(cls, path: str) -> dict: ...
    @staticmethod
    def save(path: str, data: dict) -> None: ...
    @classmethod
    def organize(cls, pathname: str) -> None: ...
    @classmethod
    def organize_internal(cls) -> None: ...
    @classmethod
    def resolve_path(cls, file_location: str) -> str: ...
    @classmethod
    def resolve_path_and_load_file(cls, file_location: str) -> dict: ...

class Specification(TypeSafeGetter):
    __SPECIFICATIONS: Final[dict]
    @classmethod
    def _get_data(cls) -> dict: ...
    @classmethod
    def get_directory(cls, category: str, *_sub: str) -> str: ...

class DataBase(TypeSafeGetter):
    __DATA_BASE_DICT: Final[dict]
    @classmethod
    def _get_data(cls) -> dict: ...
    @classmethod
    def update(cls, _value: dict) -> None: ...

class GlobalVariables(TypeSafeGetter, TypeSafeSetter):
    __GLOBAL_VARIABLES_DICT: Final[dict]
    @classmethod
    def _get_data(cls) -> dict: ...
    @classmethod
    def remove(cls, _key: str) -> None: ...
    @classmethod
    def if_get_set(cls, _key: str, valueToGet: object, valueToSet: object) -> bool: ...

class Setting(TypeSafeGetter, TypeSafeSetter):
    __SETTING_DATA: Final[dict]
    __SETTING_FILE_NAME: Final[str]
    @classmethod
    def _get_data(cls) -> dict: ...
    @classmethod
    def reload(cls) -> None: ...
    @classmethod
    def save(cls) -> None: ...
    @classmethod
    def get_font(cls) -> str: ...
    @classmethod
    def set_font(cls, font_name: str) -> None: ...
    @classmethod
    def get_font_type(cls) -> str: ...
    @classmethod
    def set_font_type(cls, font_type: str) -> None: ...
    @classmethod
    def get_antialias(cls) -> bool: ...
    @classmethod
    def get_language(cls) -> str: ...
    @classmethod
    def get_low_memory_mode(cls) -> bool: ...

class Debug:
    __ENABLE_DEVELOPER_MODE: bool
    __ENABLE_CHEATING: bool
    __SHOW_FPS: bool
    @classmethod
    def get_developer_mode(cls) -> bool: ...
    @classmethod
    def set_developer_mode(cls, value: bool) -> None: ...
    @classmethod
    def get_cheat_mode(cls) -> bool: ...
    @classmethod
    def set_cheat_mode(cls, value: bool) -> None: ...
    @classmethod
    def get_show_fps(cls) -> bool: ...
    @classmethod
    def set_show_fps(cls, value: bool) -> None: ...

class Info:
    __VERSION: Final[int]
    __REVISION: Final[int]
    __PATCH: Final[int]
    @classmethod
    def ensure_linpg_version(cls, action: str, revision: int, patch: int, version: int = 3) -> bool: ...
    @classmethod
    def get_current_version(cls) -> str: ...
    @classmethod
    def get_repository_url(cls) -> str: ...

class Files:
    @staticmethod
    def natural_sort(_files: list[str]) -> list[str]: ...
    @classmethod
    def search_and_remove_folder(cls, folder_to_search: str, stuff_to_remove: str) -> None: ...
    @staticmethod
    def delete_if_exist(path: str) -> None: ...
    @staticmethod
    def hash(path: str) -> str: ...

class Cache:
    __CACHE_FOLDER: Final[str]
    __CACHE_FILES_DATA_PATH: Final[str]
    __CACHE_FILES_DATA: Final[dict[str, dict]]
    @classmethod
    def get_directory(cls) -> str: ...
    @classmethod
    def new(cls, _key: str, source_file_path: str, target_file_path: str) -> None: ...
    @classmethod
    def remove(cls, _key: str) -> None: ...
    @classmethod
    def get_cache_path(cls, _key: str) -> str: ...
    @classmethod
    def match(cls, _key: str, source_file_path: str) -> bool: ...
